# File: cal_cam.m
# Usage: octave cal_cam.m <cal_data> <outfile> <calib_im>
#  where - 
#       cal_data = the output from Calibrate.py [x y z u v]
#	outfile =  the output file for the calibration matrix
#	calib_im = the image used to calibrate when running Calibrate.py
# Uses files: matrix_read.m, pnm_getc.m, pnm_getint.m, pnm_read.m,
#	pnm_write.m, drawline.m

if (length(argv)!=3)
     disp("Usage: octave cal_cam.m <cal_data> <outfile> <calib_im>");
     exit;
endif

match = matrixread(nth(argv,1), -1, 5);

match(:,1:3) = match(:,1:3) * 2.54;     % convert from inches to cm

Q = calibrate( match );

[outfile, msg] = fopen(nth(argv,2), "w");
fprintf( outfile, "%.10f %.10f %.10f %.10f\n", Q' );#'
fclose(outfile);

cubev = [   1   1   1;
            1   1  -1;
            1  -1   1;
            1  -1  -1;
           -1   1   1;
           -1   1  -1;
           -1  -1   1;
           -1  -1  -1 ];
cubee = [   1   2;
            2   4;
            4   3;
            3   1;
            5   6;
            6   8;
            8   7;
            7   5;
            1   5;
            2   6;
            4   8;
            3   7   ];


im = pnm_read( nth(argv,3));
[nr,nc] = size(im);
r = im(:, 1:3:nc);
g = im(:, 2:3:nc);
b = im(:, 3:3:nc);

#r = im;
#g = im;
#b = im;

cube = cubev * 32.7025;
for i = 1:12
    P1 = [ cube(cubee(i,1),:), 1] ';
    P2 = [ cube(cubee(i,2),:), 1 ]';
    p1 = Q * P1;
    p2 = Q * P2;
    u1 = round( p1(1)/p1(3) );
    v1 = round( p1(2)/p1(3) );
    u2 = round( p2(1)/p2(3) );
    v2 = round( p2(2)/p2(3) );
    
    r = drawline( r, u1, v1, u2, v2, 255 );
endfor

pnm_write( "ppm-raw", "testout.ppm", r, g, b );


    
